@@ -206,6 +206,12 @@ protected slots:
     */
     void CheckDone();
 
+    /**
+    * @brief Lock down UI while checking
+    *
+    */
+    void CheckLockDownUI();
+
     /**
     * @brief Slot for enabling save and clear button
     *
@@ -255,9 +261,9 @@ protected:
 
     /**
     * @brief Set current language
-    * @param index Index of the language to set
+    * @param code Language code of the language to set (e.g. "en").
     */
-    void SetLanguage(const int index);
+    void SetLanguage(const QString &code);
 
     /**
     * @brief Event coming when application is about to close.
@@ -330,7 +336,7 @@ protected:
     /**
     * @brief Show help contents
     */
-    void OpenHtmlHelpContents();
+    void OpenOnlineHelp();
 
     /**
     * @brief Enable or disable project file actions.
@@ -354,6 +360,18 @@ protected:
     */
     void AddIncludeDirs(const QStringList &includeDirs, Settings &result);
 
+    /**
+    * @brief Handle command line parameters given to GUI.
+    * @param params List of string given to command line.
+    */
+    void HandleCLIParams(const QStringList &params);
+
+    /**
+    * @brief Load project file to the GUI.
+    * @param filePath Filename (inc. path) of project file to load.
+    */
+    void LoadProjectFile(const QString &filePath);
+
     /**
     * @brief Program settings
     *
@@ -384,12 +402,6 @@ protected:
     */
     Ui::MainWindow mUI;
 
-    /**
-    * @brief Group holding all supported languages
-    *
-    */
-    QActionGroup *mLanguages;
-
     /**
     * @brief Current checked directory.
     */
@@ -400,11 +412,6 @@ protected:
     */
     LogView *mLogView;
 
-    /**
-     * @brief Help window..
-     */
-    HelpWindow *mHelpWindow;
-
     /**
     * @brief Project (file).
     */
