@@ -329,6 +329,31 @@ static bool for_condition(const Token * const tok2, unsigned int varid, std::str
 
 
 
+/**
+ * calculate maximum value of loop variable
+ * @param stepvalue token that contains the step value
+ * @param min_value the minimum value of loop variable
+ * @param max_value maximum value of the loop variable
+ */
+static bool for_maxvalue(const Token * const stepvalue, const std::string &min_value, std::string &max_value)
+{
+    if (!MathLib::isInt(stepvalue->str()))
+        return false;
+
+    // We have for example code: "for(i=2;i<22;i+=6)
+    // We can calculate that max value for i is 20, not 21
+    // 21-2 = 19
+    // 19/6 = 3
+    // 6*3+2 = 20
+    const MathLib::bigint num = MathLib::toLongNumber(stepvalue->str());
+    MathLib::bigint max = MathLib::toLongNumber(max_value);
+    const MathLib::bigint min = MathLib::toLongNumber(min_value);
+    max = ((max - min) / num) * num + min;
+    max_value = MathLib::toString<MathLib::bigint>(max);
+    return true;
+}
+
+
 /**
  * Parse the third sub-statement in for head
  * \param tok first token
@@ -347,55 +372,24 @@ static bool for3(const Token * const tok,
     if (Token::Match(tok, "%varid% += %num% )", varid) ||
         Token::Match(tok, "%varid%  = %num% + %varid% )", varid))
     {
-        if (!MathLib::isInt(tok->strAt(2)))
+        if (!for_maxvalue(tok->tokAt(2), min_value, max_value))
             return false;
-
-        const MathLib::bigint num = MathLib::toLongNumber(tok->strAt(2));
-
-        // We have for example code: "for(i=2;i<22;i+=6)
-        // We can calculate that max value for i is 20, not 21
-        // 21-2 = 19
-        // 19/6 = 3
-        // 6*3+2 = 20
-        MathLib::bigint max = MathLib::toLongNumber(max_value);
-        MathLib::bigint min = MathLib::toLongNumber(min_value);
-        max = ((max - min) / num) * num + min;
-        max_value = MathLib::toString<MathLib::bigint>(max);
     }
     else if (Token::Match(tok, "%varid% = %varid% + %num% )", varid))
     {
-        if (!MathLib::isInt(tok->strAt(4)))
+        if (!for_maxvalue(tok->tokAt(4), min_value, max_value))
             return false;
-
-        const MathLib::bigint num = MathLib::toLongNumber(tok->strAt(4));
-        MathLib::bigint max = MathLib::toLongNumber(max_value);
-        MathLib::bigint min = MathLib::toLongNumber(min_value);
-        max = ((max - min) / num) * num + min;
-        max_value = MathLib::toString<MathLib::bigint>(max);
     }
     else if (Token::Match(tok, "%varid% -= %num% )", varid) ||
              Token::Match(tok, "%varid%  = %num% - %varid% )", varid))
     {
-        if (!MathLib::isInt(tok->strAt(2)))
+        if (!for_maxvalue(tok->tokAt(2), min_value, max_value))
             return false;
-
-        const MathLib::bigint num = MathLib::toLongNumber(tok->strAt(2));
-
-        MathLib::bigint max = MathLib::toLongNumber(max_value);
-        MathLib::bigint min = MathLib::toLongNumber(min_value);
-        max = ((max - min) / num) * num + min;
-        max_value = MathLib::toString<MathLib::bigint>(max);
     }
     else if (Token::Match(tok, "%varid% = %varid% - %num% )", varid))
     {
-        if (!MathLib::isInt(tok->strAt(4)))
+        if (!for_maxvalue(tok->tokAt(4), min_value, max_value))
             return false;
-
-        const MathLib::bigint num = MathLib::toLongNumber(tok->strAt(4));
-        MathLib::bigint max = MathLib::toLongNumber(max_value);
-        MathLib::bigint min = MathLib::toLongNumber(min_value);
-        max = ((max - min) / num) * num + min;
-        max_value = MathLib::toString<MathLib::bigint>(max);
     }
     else if (Token::Match(tok, "--| %varid% --| )", varid))
     {
@@ -464,6 +458,20 @@ void CheckBufferOverrun::parse_for_body(const Token *tok2, const ArrayInfo &arra
                 break;
         }
 
+        // TODO: try to reduce false negatives. This is just a quick fix
+        // for TestBufferOverrun::array_index_for_question
+        if (tok2->str() == "?")
+            break;
+
+        if (Token::simpleMatch(tok2, "for (") && Token::simpleMatch(tok2->next()->link(), ") {"))
+        {
+            const Token *endpar = tok2->next()->link();
+            const Token *startbody = endpar->next();
+            const Token *endbody = startbody->link();
+            tok2 = endbody;
+            continue;
+        }
+
         if (Token::Match(tok2, "if|switch"))
         {
             if (bailoutIfSwitch(tok2, arrayInfo.varid))
@@ -758,6 +766,64 @@ void CheckBufferOverrun::checkFunctionCall(const Token *tok, const ArrayInfo &ar
 }
 
 
+void CheckBufferOverrun::checkScopeForBody(const Token *tok, const ArrayInfo &arrayInfo, bool &bailout)
+{
+    bailout = false;
+    const Token *tok2 = tok->tokAt(2);
+    const MathLib::bigint size = arrayInfo.num[0];
+
+    // Check if there is a break in the body..
+    {
+        const Token *bodyStart = tok->next()->link()->next();
+        const Token *bodyEnd = bodyStart->link();
+        if (Token::findmatch(bodyStart, "break ;", bodyEnd))
+            return;
+    }
+
+    unsigned int counter_varid = 0;
+    std::string min_counter_value;
+    std::string max_counter_value;
+
+    tok2 = for_init(tok2, counter_varid, min_counter_value);
+    if (tok2 == 0 || counter_varid == 0)
+        return;
+
+    bool maxMinFlipped = false;
+    std::string strindex;
+    if (!for_condition(tok2, counter_varid, min_counter_value, max_counter_value, strindex, maxMinFlipped))
+        return;
+
+    // Get index variable and stopsize.
+    bool condition_out_of_bounds = bool(size > 0);
+    if (MathLib::toLongNumber(max_counter_value) < size)
+        condition_out_of_bounds = false;
+
+    if (!for3(tok2->tokAt(4), counter_varid, min_counter_value, max_counter_value, maxMinFlipped))
+        return;
+
+    if (Token::Match(tok2->tokAt(4), "%var% =|+=|-=") && MathLib::toLongNumber(max_counter_value) <= size)
+        condition_out_of_bounds = false;
+
+    // Goto the end parenthesis of the for-statement: "for (x; y; z)" ..
+    tok2 = tok->next()->link();
+    if (!tok2 || !tok2->tokAt(5))
+    {
+        bailout = true;
+        return;
+    }
+
+    // Check is the counter variable increased elsewhere inside the loop or used
+    // for anything else except reading
+    if (for_bailout(tok2->next(), counter_varid))
+    {
+        bailout = true;
+        return;
+    }
+
+    parse_for_body(tok2->next(), arrayInfo, strindex, condition_out_of_bounds, counter_varid, min_counter_value, max_counter_value);
+}
+
+
 void CheckBufferOverrun::checkScope(const Token *tok, const std::vector<std::string> &varname, const MathLib::bigint size, const MathLib::bigint total_size, unsigned int varid)
 {
     std::string varnames;
@@ -871,53 +937,11 @@ void CheckBufferOverrun::checkScope(const Token *tok, const std::vector<std::str
         // Loop..
         if (Token::simpleMatch(tok, "for ("))
         {
-            const Token *tok2 = tok->tokAt(2);
-
-            // Check if there is a break in the body..
-            {
-                const Token *bodyStart = tok->next()->link()->next();
-                const Token *bodyEnd = bodyStart->link();
-                if (Token::findmatch(bodyStart, "break ;", bodyEnd))
-                    continue;
-            }
-
-            unsigned int counter_varid = 0;
-            std::string min_counter_value;
-            std::string max_counter_value;
-
-            tok2 = for_init(tok2, counter_varid, min_counter_value);
-            if (tok2 == 0 || counter_varid == 0)
-                continue;
-
-            bool maxMinFlipped = false;
-            std::string strindex;
-            if (!for_condition(tok2, counter_varid, min_counter_value, max_counter_value, strindex, maxMinFlipped))
-                continue;
-
-            // Get index variable and stopsize.
-            bool condition_out_of_bounds = true;
-            if (MathLib::toLongNumber(max_counter_value) < size)
-                condition_out_of_bounds = false;
-
-            if (!for3(tok2->tokAt(4), counter_varid, min_counter_value, max_counter_value, maxMinFlipped))
-                continue;
-
-            if (Token::Match(tok2->tokAt(4), "%var% =|+=|-=") && MathLib::toLongNumber(max_counter_value) <= size)
-                condition_out_of_bounds = false;
-
-            // Goto the end parenthesis of the for-statement: "for (x; y; z)" ..
-            tok2 = tok->next()->link();
-            if (!tok2 || !tok2->tokAt(5))
+            const ArrayInfo arrayInfo(varid, varnames, (unsigned int)size, (unsigned int)total_size);
+            bool bailout = false;
+            checkScopeForBody(tok, arrayInfo, bailout);
+            if (bailout)
                 break;
-
-            // Check is the counter variable increased elsewhere inside the loop or used
-            // for anything else except reading
-            if (for_bailout(tok2->next(), counter_varid))
-                break;
-
-            ArrayInfo arrayInfo(varid, varnames, (unsigned int)size, (unsigned int)total_size);
-            parse_for_body(tok2->next(), arrayInfo, strindex, condition_out_of_bounds, counter_varid, min_counter_value, max_counter_value);
-
             continue;
         }
 
@@ -971,7 +995,7 @@ void CheckBufferOverrun::checkScope(const Token *tok, const std::vector<std::str
         }
 
         // Check function call..
-        if (Token::Match(tok, "%var% ("))
+        if (Token::Match(tok, "%var% (") && total_size > 0)
         {
             // No varid => function calls are not handled
             if (varid == 0)
@@ -1069,52 +1093,10 @@ void CheckBufferOverrun::checkScope(const Token *tok, const ArrayInfo &arrayInfo
         // Loop..
         else if (Token::simpleMatch(tok, "for ("))
         {
-            const Token *tok2 = tok->tokAt(2);
-
-            // Check if there is a break in the body..
-            {
-                const Token *bodyStart = tok->next()->link()->next();
-                const Token *bodyEnd = bodyStart->link();
-                if (Token::findmatch(bodyStart, "break ;", bodyEnd))
-                    continue;
-            }
-
-            unsigned int counter_varid = 0;
-            std::string min_counter_value;
-            std::string max_counter_value;
-
-            tok2 = for_init(tok2, counter_varid, min_counter_value);
-            if (tok2 == 0 || counter_varid == 0)
-                continue;
-
-            bool maxMinFlipped = false;
-            std::string strindex;
-            if (!for_condition(tok2, counter_varid, min_counter_value, max_counter_value, strindex, maxMinFlipped))
-                continue;
-
-            // Get index variable and stopsize.
-            bool condition_out_of_bounds = true;
-            if (MathLib::toLongNumber(max_counter_value) < (int)arrayInfo.num[0])
-                condition_out_of_bounds = false;
-
-            if (!for3(tok2->tokAt(4), counter_varid, min_counter_value, max_counter_value, maxMinFlipped))
-                continue;
-
-            if (Token::Match(tok2->tokAt(4), "%var% =|+=|-=") && MathLib::toLongNumber(max_counter_value) <= (int)arrayInfo.num[0])
-                condition_out_of_bounds = false;
-
-            // Goto the end parenthesis of the for-statement: "for (x; y; z)" ..
-            tok2 = tok->next()->link();
-            if (!tok2 || !tok2->tokAt(5))
+            bool bailout = false;
+            checkScopeForBody(tok, arrayInfo, bailout);
+            if (bailout)
                 break;
-
-            // Check is the counter variable increased elsewhere inside the loop or used
-            // for anything else except reading
-            if (for_bailout(tok2->next(), counter_varid))
-                break;
-
-            parse_for_body(tok2->next(), arrayInfo, strindex, condition_out_of_bounds, counter_varid, min_counter_value, max_counter_value);
-
             continue;
         }
 
@@ -1426,7 +1408,7 @@ void CheckBufferOverrun::checkStructVariable()
         const std::string &structname = tok->next()->str();
         const Token *tok2 = tok;
 
-        while (tok2->str() != "{")
+        while (tok2 && tok2->str() != "{")
             tok2 = tok2->next();
 
         // Found a struct declaration. Search for arrays..
@@ -1696,7 +1678,7 @@ void CheckBufferOverrun::checkSprintfCall(const Token *tok, const MathLib::bigin
                 // and skip to next token.
                 parameters.push_back(0);
 
-                // count parantheses for tok3
+                // count parentheses for tok3
                 int ind = 0;
                 for (const Token *tok3 = tok2->next(); tok3; tok3 = tok3->next())
                 {
@@ -1987,6 +1969,9 @@ bool CheckBufferOverrun::ArrayInfo::declare(const Token *tok, const Tokenizer &t
     _element_size = 0;
     _varname.clear();
 
+    if (!tok)
+        return false;
+
     if (!tok->isName() || tok->str() == "return")
         return false;
 
@@ -2041,6 +2026,9 @@ bool CheckBufferOverrun::ArrayInfo::declare(const Token *tok, const Tokenizer &t
             atok = atok->tokAt(2);
     }
 
+    if (Token::Match(atok, "] = !!{"))
+        return false;
+
     return (!_num.empty() && Token::Match(atok, "] ;|="));
 }
 
