@@ -64,6 +64,8 @@ public:
         checkOther.checkAssignmentInAssert();
         checkOther.checkSizeofForArrayParameter();
         checkOther.checkSelfAssignment();
+
+        checkOther.clarifyCondition();   // not simplified because ifAssign
     }
 
     /** @brief Run checks against the simplified token list */
@@ -87,11 +89,19 @@ public:
         checkOther.checkMisusedScopedObject();
         checkOther.checkCatchExceptionByValue();
         checkOther.checkMemsetZeroBytes();
+        checkOther.checkIncorrectStringCompare();
+        checkOther.checkIncrementBoolean();
+        checkOther.checkComparisonOfBoolWithInt();
+        checkOther.checkSwitchCaseFallThrough();
     }
 
     /** @brief Clarify calculation for ".. a * b ? .." */
     void clarifyCalculation();
-    void clarifyCalculationError(const Token *tok);
+    void clarifyCalculationError(const Token *tok, const std::string &op);
+
+    /** @brief Suspicious condition (assignment+comparison) */
+    void clarifyCondition();
+    void clarifyConditionError(const Token *tok);
 
     /** @brief Are there C-style pointer casts in a c++ file? */
     void warningOldStylePointerCast();
@@ -159,6 +169,9 @@ public:
     /** @brief %Check for assigning to the same variable twice in a switch statement*/
     void checkRedundantAssignmentInSwitch();
 
+    /** @brief %Check for switch case fall through without comment */
+    void checkSwitchCaseFallThrough();
+
     /** @brief %Check for assigning a variable to itself*/
     void checkSelfAssignment();
 
@@ -180,6 +193,15 @@ public:
     /** @brief %Check for using sizeof with array given as function argument */
     void checkSizeofForArrayParameter();
 
+    /** @brief %Check for using bad usage of strncmp and substr */
+    void checkIncorrectStringCompare();
+
+    /** @brief %Check for using postfix increment on bool */
+    void checkIncrementBoolean();
+
+    /** @brief %Check for suspicious comparison of a bool and a non-zero (and non-one) value (e.g. "if (!x==4)") */
+    void checkComparisonOfBoolWithInt();
+
     // Error messages..
     void cstyleCastError(const Token *tok);
     void dangerousUsageStrtolError(const Token *tok);
@@ -197,6 +219,7 @@ public:
     void mathfunctionCallError(const Token *tok, const unsigned int numParam = 1);
     void fflushOnInputStreamError(const Token *tok, const std::string &varname);
     void redundantAssignmentInSwitchError(const Token *tok, const std::string &varname);
+    void switchCaseFallThrough(const Token *tok);
     void selfAssignmentError(const Token *tok, const std::string &varname);
     void assignmentInAssertError(const Token *tok, const std::string &varname);
     void incorrectLogicOperatorError(const Token *tok);
@@ -204,6 +227,9 @@ public:
     void catchExceptionByValueError(const Token *tok);
     void memsetZeroBytesError(const Token *tok, const std::string &varname);
     void sizeofForArrayParameterError(const Token *tok);
+    void incorrectStringCompareError(const Token *tok, const std::string& func, const std::string &string, const std::string &len);
+    void incrementBooleanError(const Token *tok);
+    void comparisonOfBoolWithIntError(const Token *tok, const std::string &varname);
 
     void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings)
     {
@@ -232,6 +258,7 @@ public:
         c.sizeofsizeofError(0);
         c.sizeofCalculationError(0);
         c.redundantAssignmentInSwitchError(0, "varname");
+        c.switchCaseFallThrough(0);
         c.selfAssignmentError(0, "varname");
         c.assignmentInAssertError(0, "varname");
         c.invalidScanfError(0);
@@ -242,7 +269,11 @@ public:
         c.unassignedVariableError(0, "varname");
         c.catchExceptionByValueError(0);
         c.memsetZeroBytesError(0, "varname");
-        c.clarifyCalculationError(0);
+        c.clarifyCalculationError(0, "+");
+        c.clarifyConditionError(0);
+        c.incorrectStringCompareError(0, "substr", "\"Hello World\"", "12");
+        c.incrementBooleanError(0);
+        c.comparisonOfBoolWithIntError(0, "varname");
     }
 
     std::string myName() const
@@ -261,6 +292,7 @@ public:
                "* scoped object destroyed immediately after construction\n"
                "* assignment in an assert statement\n"
                "* sizeof for array given as function argument\n"
+               "* incorrect length arguments for 'substr' and 'strncmp'\n"
 
                // style
                "* C-style pointer cast in cpp file\n"
@@ -281,7 +313,10 @@ public:
                "* assignment of a variable to itself\n"
                "* mutual exclusion over || always evaluating to true\n"
                "* exception caught by value instead of by reference\n"
-               "* Clarify calculation with parantheses\n"
+               "* Clarify calculation with parentheses\n"
+               "* using increment on boolean\n"
+               "* comparison of a boolean with a non-zero integer\n"
+               "* suspicious condition (assignment+comparison)"
 
                // optimisations
                "* optimisation: detect post increment/decrement\n";
