@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2010 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2011 Daniel Marjamäki and Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,12 +33,12 @@ class CheckUnusedFunctions: public Check
 {
 public:
     /** @brief This constructor is used when registering the CheckUnusedFunctions */
-    CheckUnusedFunctions() : Check()
+    CheckUnusedFunctions() : Check(myName())
     { }
 
     /** @brief This constructor is used when running checks. */
     CheckUnusedFunctions(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger)
-        : Check(tokenizer, settings, errorLogger)
+        : Check(myName(), tokenizer, settings, errorLogger)
     { }
 
     // Parse current tokens and determine..
@@ -50,9 +50,10 @@ public:
 
 private:
 
-    void getErrorMessages()
+    void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings)
     {
-        unusedFunctionError(0, "", "funcName");
+        CheckUnusedFunctions c(0, settings, errorLogger);
+        c.unusedFunctionError(errorLogger, "", "funcName");
     }
 
     /**
@@ -68,7 +69,7 @@ private:
 
     }
 
-    std::string name() const
+    std::string myName() const
     {
         return "Unused functions";
     }
