@@ -39,6 +39,7 @@ private:
         TEST_CASE(test3);
         TEST_CASE(test4);
         TEST_CASE(test5);
+        TEST_CASE(test6); // ticket #2602
 
         // [ 2236547 ] False positive --style unused function, called via pointer
         TEST_CASE(func_pointer1);
@@ -65,6 +66,9 @@ private:
         TEST_CASE(testDoesNotIdentifyMethodAsFirstFunctionArgument); // #2480
         TEST_CASE(testDoesNotIdentifyMethodAsMiddleFunctionArgument);
         TEST_CASE(testDoesNotIdentifyMethodAsLastFunctionArgument);
+
+        TEST_CASE(multiFile);
+        TEST_CASE(unknownBaseTemplate); // ticket #2580
     }
 
 
@@ -222,6 +226,14 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void test6() // ticket #2602 segmentation fault
+    {
+        check("class A {\n"
+              "    A& operator=(const A&);\n"
+              "};\n");
+        ASSERT_EQUALS("", errout.str());
+    }
+
 
 
 
@@ -549,6 +561,40 @@ private:
              );
         ASSERT_EQUALS("", errout.str());
     }
+
+    void multiFile() // ticket #2567
+    {
+        check("#file \"test.h\"\n"
+              "struct Fred\n"
+              "{\n"
+              "    Fred()\n"
+              "    {\n"
+              "        Init();\n"
+              "    }\n"
+              "private:\n"
+              "    void Init();\n"
+              "};\n"
+              "#endfile\n"
+              "void Fred::Init()\n"
+              "{\n"
+              "}\n");
+
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void unknownBaseTemplate() // ticket #2580
+    {
+        check("class Bla : public Base2<Base> {\n"
+              "public:\n"
+              "    Bla() {}\n"
+              "private:\n"
+              "    virtual void F() const;\n"
+              "};\n"
+              "void Bla::F() const { }");
+
+        ASSERT_EQUALS("", errout.str());
+    }
+
 };
 
 REGISTER_TEST(TestUnusedPrivateFunction)
